# -*- coding: utf-8 -*-

from pyramid.config import Configurator
from pyramid.i18n import TranslationStringFactory
from pyramid_beaker import session_factory_from_settings
from ringo.lib.sql.db import setup_db_session, setup_db_engine
from ringo.lib.helpers import get_action_routename
from ringo.lib.sitetree import site_tree_branches
from ringo.model import Base
from ringo.lib.i18n import translators
from ringo.lib.i18n import locale_negotiator
from ringo.resources import get_resource_factory
from efa.model.participant import Participant
from efa.model.participation import Participation


sitetree = {
    "participants": {},
    "participations": {"parent_site": "participants",
                       "parent_item": "participant",
                       "display_item": "project",
                       "display_format": u"Teilnahme im Projekt „{item}“",
                       "actions": {"info-esf-rules": u"ESF-Validierung",
                                   "journallist": u"Tagebuch"}},
    "journals": {"parent_site": "participations",
                 "display_item": "participation",
                 "display_format": "Tagebucheintrag",
                 "parent_item": "participation"}
}


site_tree_branches.append(sitetree)


def main(global_config, **settings):
    """ This function returns a Pyramid WSGI application.
    """
    engine = setup_db_engine(settings)
    setup_db_session(engine, settings)
    Base.metadata.bind = engine
    config = Configurator(settings=settings,
                          locale_negotiator=locale_negotiator)
    config.set_session_factory(session_factory_from_settings(settings))
    # Include basic ringo configuration.
    config.include('ringo')
    config.include('efa')
    config.scan()
    return config.make_wsgi_app()


def includeme(config):
    # Now configure the application and optionally overwrite previously
    translators.append(TranslationStringFactory('efa'))
    config.add_translation_dirs('efa:locale/')
    config.add_static_view('efa-static', path='efa:static',
                           cache_max_age=3600)
    config.add_route(get_action_routename(Participant, 'takeup'),
                     'participants/takeup/{id}',
                     factory=get_resource_factory(Participant))
    config.add_route('searchdisableuser', '/participants/search/disabled')
    config.add_route('info-cm-oi', '/participations/info_cm')
    config.add_route('info-8h-oi', '/participations/info_8h_oi')
    config.add_route('controllist', '/participations/controllist')
    config.add_route('report_sachbericht', '/participations/reports/sachbericht')
    config.add_route('report_durchfuehrungsbericht', '/participations/reports/durchfuehrungsbericht')
    config.add_route('participations-journallist',
                     '/participations/journallist/{id}',
                     factory=get_resource_factory(Participation))
    config.add_route('participations-excelexport',
                     '/participations/excelexport')
    config.add_route('participants-excelexport',
                     '/participants/excelexport')
    config.add_route('frueherkennung', '/frueherkennung')
    config.add_route('frueherkennung_fp2', '/frueherkennung_fp2')
