# -*- coding: utf-8 -*-

from ringo.lib.helpers import HTML
from efa.model.participation import calc_complete
from ringo.lib.sql import DBSession

def exist_complete_and_valid_oi_in(participations, no_recalc=False):
    """Check if one of the participations is valid and complete"""
    for participation in participations:
        if participation.valid and check_oi_complete(participation, no_recalc):
            return True
    return False

def check_oi_complete(participation, no_recalc=False):
    if no_recalc:
        return participation.oi and participation.complete_oi == 100
    else:
        return participation.oi and int(calc_complete(
            participation).split(',')[0]) == 100

def render_esf_errors(participation,
                      klass, OK, FEHLER,
                      NICHTAUSWERTBAR):
    out = []
    if participation.oi:
        if participation.complete_oi == 100 \
                and participation.is_valid():
            out.append(OK)
        elif not participation.is_valid():
            out.append(FEHLER)
        else:
            # not complete and valid case
            out.append(NICHTAUSWERTBAR)
    else:
        if participation.complete_oi != 100:
            out.append(NICHTAUSWERTBAR)
        elif not participation.start_date:
            out.append(NICHTAUSWERTBAR)
        else:
            participations = DBSession.query(klass) \
                .filter(klass.participant_id == \
                        participation.participant_id) \
                .filter(klass.start_date <= \
                        participation.start_date).all()
            before_complete_and_valid = exist_complete_and_valid_oi_in(
                participations, no_recalc=True)
            if before_complete_and_valid:
                out.append(OK)
            else:
                out.append(NICHTAUSWERTBAR)
    return out
