from datetime import date
from efa.lib.whitelist import Whitelist
from efa.scripts.participations import ReportConfig, get_whitelisted_virtual_participations


def filter_virtual_participation(result, filters):
    date_filters = {
        "entry_in": entry_in,
        "exit_in": exit_in,
        "entry_and_exit_in": entry_and_exit_in,
        "entry_or_exit_in": entry_or_exit_in,
        "running_while": running_while,
        "no_filter": no_filter
    }

    gender_filters = {
        "only_male": lambda x: x["gender"] == 1,
        "only_female": lambda x: x["gender"] == 0,
        "both": lambda x: x
    }
    foerderphase_filters = {
        "only_fp1": lambda x: x.get("start_date") < date(2019, 1, 1),
        "only_fp2": lambda x: x.get("start_date") > date(2018, 12, 31),
        "both": lambda x: x
    }

    selected_date_filter = date_filters[filters.entry_filter]
    gender_filter = gender_filters[filters.gender_filter]
    foerderphase_filter = foerderphase_filters[filters.foerderphase]
    return [participation for participation in result
            if selected_date_filter(filters.start, filters.end,
                                    participation)
            and gender_filter(participation)
            and foerderphase_filter(participation)]


def running_while(start, end, participation):
    """
    Business logic see Issue2676
    :param start: start of the interval
    :param end: end of the interval
    :param participation: participation to be filtered
    :return: boolean
    """
    return (participation["start_date"] <= end)\
           and (participation["start_date"] is None or participation["start_date"]>=start)

def entry_in(start, end, participation):
    return participation["start_date"] >= start and participation["start_date"] <= end


def exit_in(start, end, participation):
    if participation.get("end_date"):
        return participation["end_date"] >= start and participation["end_date"] <= end
    return False


def entry_and_exit_in(start, end, participation):
    return entry_in(start, end, participation) and exit_in(start, end, participation)


def entry_or_exit_in(start, end, participation):
    return entry_in(start, end, participation) or exit_in(start, end, participation)


def no_filter(start, end, participation):
    return True


def whitelist(fields, result):
    processed = []
    for row in result:
        processed.append([row.get(field) for field in fields])
    return processed


def get_whitelisted_virtual_participations_for(request, app, Participation, report, start, end):
    report_config = ReportConfig(app, Participation, report, start, end)
    whitelist = Whitelist.from_file(request.registry.settings.get("whitelist_path"))
    return get_whitelisted_virtual_participations(whitelist, report_config,
                                                  request.db)
