# -*- coding: utf-8 -*-

from datetime import date
from openpyxl import Workbook
from openpyxl import load_workbook
from openpyxl.writer.excel import save_virtual_workbook


def get_template_file_by_request(request, exportname, simple_filenames):
    if not simple_filenames:
        template_file = request.registry.settings.get(
                "efa.participant_template")
        if template_file is None:
            raise ValueError("efa.participant_template not set.")
    else:
        template_file = request.registry.settings.get(
                    "efa.default_template")
        if template_file is None:
            raise ValueError("efa.default_template not set.")

    return template_file


def get_filename(program, exportname, simple_filenames, extension=""):

    today = str(date.today().strftime("%Y-%m-%d"))
    program = program.upper()

    template = "{}_{}_Auswertungsvorlage_{}"
    if simple_filenames:
        template = "{}_{}_{}"

    if extension is not "":
        template = template + ".{}"

    return template.format(program, exportname, today, extension)


def decode(value):
    if isinstance(value, str):
        return unicode(value, "utf-8")
    return value


def write_excel(ws, data):
    for row_num, row_data in enumerate(data, start=1):
        for col_num, col_data in enumerate(row_data, start=1):
            ws.cell(row=row_num, column=col_num).value = decode(col_data)

def export_xlsx_template(request, rows, exportname, program, metadata,
                         simple_filenames=False):

    result_filename = get_filename(program, exportname, simple_filenames, "xlsx")

    template_file = get_template_file_by_request(request, exportname,
                                                 simple_filenames)

    wb = load_workbook(filename=template_file)

    write_excel(wb["Metadaten"], metadata)
    write_excel(wb["Rohdaten"], rows)


    response = request.response
    response.body = save_virtual_workbook(wb)
    response.content_type = "application/vnd.ms-excel"
    response.content_disposition = 'attachment; filename="%s"' % result_filename

    return response


