import os
import cgi
import pkg_resources
from mako.lookup import TemplateLookup
from formbar.renderer import FieldRenderer

import ringo.lib.helpers
from ringo.views.helpers import get_item_from_request

from ringo.lib.renderer.lists import DTListRenderer
from ringo.lib.renderer import (
    DialogRenderer,
    ExportDialogRenderer
)
import ringo.lib.security as security
from ringo.lib.helpers import HTML, literal, get_item_actions
from ringo.lib.form import (
    get_eval_url
)

base_dir = pkg_resources.get_distribution("efa").location
template_dir = os.path.join(base_dir, 'efa', 'templates')
template_lookup = TemplateLookup(directories=[template_dir],
                                 default_filters=['h'])


def user_list_options_html(user, users):
        html = []
        html.append('<select name="coworker" class="form-control">')
        html.append("<options>")
        for x in users:
            if x.id != user.id:
                profile = cgi.escape(unicode(x.profile[0]))
                html.append('<option value="%s">' % x.id)
                html.append(profile)
                html.append('</option>')

        html.append("</options>")
        html.append("</select>")
        return html


class ParticipationCompletenessRenderer():

    def __init__(self, complete):
        self.complete = complete

    def render(self):
        """This function will render a progressbar as indicator on how
        complete is the case? Means how many of the required fields are
        filled?
        """
        # Check if the case is valid
        pb_css = ["progress-bar", "progress-bar-info"]
        # Render the progress bar.
        out = []
        out.append(HTML.tag("div", class_="progress", _closed=False,
            style="margin-bottom: 0px;margin-right: 20px"))
        attr = {"class_": "  ".join(pb_css),
                "role": "progressbar",
                "aria-valuemax": "100",
                "aria-valuemin": "0",
                "aria-valuenow": "%s" % self.complete,
                "style": ("width: %s%%;"
                          "text-align: left;"
                          "padding-left: 3px;" % self.complete)
                }
        out.append(HTML.tag("div", _closed=False, **attr))
        out.append(HTML.tag("span",
                            c="%s%%" % self.complete,
                            style=("color:black;"
                                   "font-size: smaller;"
                                   "text-align: left;")))
        out.append(HTML.tag("/div", _closed=False))
        out.append(HTML.tag("/div", _closed=False))
        return literal("").join(out)


class ParticipantSearchResultRenderer(DTListRenderer):

    def __init__(self, listing):
        super(ParticipantSearchResultRenderer, self).__init__(listing)
        self.template = template_lookup.get_template("participant/"
                                                     "searchresult.mako")
    def render(self, request, create_value=None):
        # Get the bundles_actions.
        # If bundle_actions is True, display checkboxes and more in the table.
        bundled_actions = []
        for action in get_item_actions(request, self.listing.clazz):
            if action.bundle and security.has_permission(action.name.lower(),
                                                         request.context,
                                                         request):
                bundled_actions.append(action)
        table_id = self._get_table_id()
        table_config = self._render_js_config(request, table_id, bundled_actions=bundled_actions)
        values = {'items': self.listing.items,
                  'clazz': self.listing.clazz,
                  'listing': self.listing,
                  'request': request,
                  '_': request.translate,
                  's': security,
                  'h': ringo.lib.helpers,
                  'bundled_actions': bundled_actions,
                  'tableconfig': self.config,
                  'tableid': table_id,
                  'dtconfig': table_config,
                  'create_value': create_value}
        return literal(self.template.render(**values))


class SingleParticipationTransferRenderer(ExportDialogRenderer):
    def __init__(self, request, clazz):
        DialogRenderer.__init__(self, request, clazz, "transfer")
        self.template = template_lookup.get_template("internal/single_transfer.mako")

    def render(self, items):
        _ = self._request.translate
        values = {}
        values['request'] = self._request
        values['items'] = items
        values['item'] = self._item
        values['body'] = self._render_body()
        values['action'] = _(self._action.capitalize())
        values['ok_url'] = self._request.current_route_path()
        values['_'] = self._request.translate
        values['cancel_url'] = self._request.referrer
        values['eval_url'] = self._request.application_url+get_eval_url()
        return literal(self.template.render(**values))

    def _render_body(self):
        # The output of the form render function of formbar is
        # considered safe here as formbar already handles the escapeing.
        # So we put it into the literal function to mark it save.
        #params = handle_params(request)
        request = self._request
        usr = request.user
        organisation = usr.organisation
        users = organisation[0].users
        html = user_list_options_html(usr, users)
        return literal("".join(html))


class ParticipationTransferRenderer(ExportDialogRenderer):
    def __init__(self, request, clazz):
        """@todo: to be defined """
        DialogRenderer.__init__(self, request, clazz, "transfer")
        self.template = template_lookup.get_template("internal/transfer.mako")

    def render(self, items):
        _ = self._request.translate
        values = {}
        values['request'] = self._request
        values['items'] = items
        values['item'] = self._item
        values['body'] = self._render_body()
        values['action'] = _(self._action.capitalize())
        values['ok_url'] = self._request.current_route_path()
        values['_'] = self._request.translate
        values['cancel_url'] = self._request.referrer
        values['eval_url'] = self._request.application_url+get_eval_url()
        return literal(self.template.render(**values))

    def _render_body(self):
        # The output of the form render function of formbar is
        # considered safe here as formbar already handles the escapeing.
        # So we put it into the literal function to mark it save.
        #params = handle_params(request)
        request = self._request
        usr = get_item_from_request(request)
        organisation = usr.organisation
        users = organisation[0].users
        html = user_list_options_html(usr, users)
        return literal("".join(html))


class ValidationInfoRenderer(FieldRenderer):

    def __init__(self, field, transfer):
        super(ValidationInfoRenderer, self).__init__(field, transfer)
        self.template = template_lookup.get_template(
            "participations/validationinfo.mako")

    def _get_template_values(self):
        values = super(ValidationInfoRenderer, self)._get_template_values()
        values["validation"] = self._field._form._item.validate()
        return values

    def _render_label(self):
        return ""
