from ConfigParser import ConfigParser
from collections import OrderedDict
from itertools import chain


class Whitelist(object):
    sections = ("beneficiarys",
                "projects",
                "participants",
                "participations",
                "outputindicators",
                "resultindicators"
                )

    allowed_special_fields = ("age",
                              "participant_uuid",
                              "completeness_entry",
                              "completeness_exit",
                              "beneficiary_id",
                              "beneficiary_name",
                              "nuts1",
                              "regioncategory_id",
                              "organisation_name",
                              "project_name",
                              "year_of_entry",
                              "year_of_exit",
                              "info_email",
                              "originating_beneficiary_id"
                              )

    @classmethod
    def from_file(cls, path, include_updated=False):
        config = ConfigParser(allow_no_value=True, dict_type=OrderedDict)
        config.readfp(open(path))
        return cls(config, include_updated)

    def __init__(self, config, include_updated):
        self._config = config
        self._include_updated = include_updated
        fields = [self._get_fields(s) for s in self.sections]
        fields.extend([[field for field in self._get_fields("special fields")
                        if field in self.allowed_special_fields]])
        self._fields = list(chain(*fields))

    @property
    def config(self):
        return self._config

    @property
    def fields(self):
        return self._fields[:]

    @property
    def fields_for_export(self):
        fields = self.fields
        upcase = lambda x: x.upper()
        fields.extend(map(upcase, self.indicators))
        return fields

    @property
    def fields_for_copy_from(self):
        fields = self.fields

        def sanify_fieldname(fieldname):
            return fieldname.lower().replace(".", "_")

        fields.extend(map(sanify_fieldname, self.indicators))
        return fields

    @property
    def indicators(self):
        return self._get_fields("indicators")

    @property
    def beneficiarys(self):
        return self._get_fields("beneficiarys")

    @property
    def projects(self):
        return self._get_fields("projects")

    @property
    def participants(self):
        return self._get_fields("participants")

    @property
    def participations(self):
        return self._get_fields("participations")

    @property
    def outputindicators(self):
        return self._get_fields("outputindicators")

    @property
    def resultindicators(self):
        return self._get_fields("resultindicators")

    @property
    def validation_rules(self):
        return self._get_fields("validation result")

    @property
    def bausteine(self):
        return [field for field in self._get_fields("special fields")
                if field in {"baustein_cms", "baustein_clearings",
                             "baustein_jsas", "baustein_mps"}]

    def apply_for_export(self, o):
        return {field:o.get(field) for field in self.fields_for_export}

    def apply_for_copy_from(self, o):
        return {field:o.get(field) for field in self.fields_for_copy_from}

    def _get_fields(self, section):
        res = self._config.items(section)
        if self._include_updated:
            return [col[0] for col in res]
        return [col[0] for col in res if col[0] not in {"updated", "created"}]

