import datetime
import sqlalchemy as sa
from ringo.model import Base
from ringo.model.base import BaseItem
from ringo.model.mixins import Owned


class Journal(BaseItem, Owned, Base):
    """A Journal is used to track the work within a
    :class:`.WaskiqParticipation`. Every user who works with the
    participant within a participation can track what he has has done.
    Beside a short description the duration and date (and time) of the
    "event" can be tracked."""
    __tablename__ = 'journals'
    _modul_id = 2004
    id = sa.Column(sa.Integer, primary_key=True)
    date = sa.Column('date', sa.Date)
    """Date and time of the journal entry. This date refers to the time
    and date when the event happened which is described by this entry."""
    duration = sa.Column('duration',
                         sa.Interval,
                         nullable=False,
                         default=datetime.timedelta())
    """The duration of the event"""
    description = sa.Column('description', sa.String, nullable=True,
                            default='')
    """Latest editor of entry"""
    editor = sa.Column('editor', sa.String, default='')
    """A short description description of the event."""
    participation_id = sa.Column('participation_id',
                                 sa.ForeignKey("participations.id"),
                                 nullable=True)
    # enable_typechecks=False to allow any subtype to be accepted for flush
    participation = sa.orm.relation("Participation", enable_typechecks=False)
