import sqlalchemy as sa
from ringo.model.mixins import Mixin


class Resubmittable(Mixin):
    """
    A mixin to handle resubmissions.
    As a first step there is the property of having errors implemented,
    as having errors is a reason for resubmission.
    Later more reasons could be added transparently
    behind the method to_resubmit and has_errors would be
    refactored to an external table
    """
    has_errors = sa.Column('has_errors', sa.BOOLEAN, default=False)

    def reset_resubmission(self):
        """Will set the values of all reasons which can cause a
        resubmission to unknown"""
        self.has_errors = None

    @property
    def to_resubmit(self):
        return self.has_errors
