import sqlalchemy as sa
from ringo.model import Base
from ringo.model.base import BaseItem
from ringo.model.mixins import (
    Owned
)


class ResultIndicators(BaseItem, Owned, Base):
    """Result indicators describe the characteristics and situation of
    participants on the day that they leave an operation.

    *Common indicators*:
    The indicators will be assigned to a specific topic. The attributes
    are prefixed with the following strings:

    """
    __tablename__ = 'resultindicators'
    _modul_id = 1007
    id = sa.Column(sa.Integer,
                   sa.ForeignKey("participations.id"),
                   primary_key=True)

    participant_id = sa.Column(sa.Integer,
                               sa.ForeignKey("participants.id"),
                               unique=True)

    # Setting cascade to custom value to avoid ISE when triggering
    # See stackoverflow.com/questions/23699651/
    participation = sa.orm.relation("Participation",
                                    enable_typechecks=False,
                                    backref=sa.orm.backref("ri",
                                                           uselist=False,
                                                           cascade="save-update, merge, delete, delete-orphan"))
    """Relation to the Participation. If the participation is deleted
    the depending outputindicatotrs are deleted too. Also disable
    typechecks here to allow subclasses of Participation to be linked
    with the Outputindicator."""

    participant = sa.orm.relation("Participant",
                                  enable_typechecks=False,
                                  backref=sa.orm.backref("ri",
                                                         uselist=False))
    """1:1 Relation to the Participant. This is only set if the
    indicators has been selected to be the relevant indicators for ESF
    evaluation of the participant. The process of selection is operation
    specific."""
    
    status_end_measure = sa.Column('status_end_measure', sa.Integer)
    new_job_search = sa.Column('new_job_search', sa.Integer)
    school_job_education = sa.Column('school_job_education', sa.Integer)
    new_qualification = sa.Column('new_qualification', sa.Integer)
    new_job_or_self_employed = sa.Column('new_job_or_self_employed',
                                         sa.Integer)
