#!/usr/bin/env python
# -*- coding: utf-8 -*-
import datetime
import os
import transaction
import warnings
from sqlalchemy import exc as sa_exc
from ringo.lib.helpers import get_app_location
from ringo.scripts.db import get_session

from efa.model.participation import Participation, \
    all_complete_and_valid_condition


def add_autoarchiver_parser(subparsers, parent):
    parser = subparsers.add_parser('autoarchiver',
                                   help='Autoarchiver administation',
                                   parents=[parent])
    subparser = parser.add_subparsers(help='Autoarchiver command help')

    setstate_parser = subparser.add_parser('setstate',
                                           help=('set state for all archivable'
                                                 ' participations'),
                                           parents=[parent])
    setstate_parser.add_argument('--days', dest='days',
                                 help=('Days to be passed before a valid and '
                                       'complete participation become '
                                       'archivable.'),
                                 type=int,
                                 default=42)
    setstate_parser.add_argument('--execute', dest="execute",
                                 action="store_true",
                                 help="Really do the setting of the state.")
    setstate_parser.set_defaults(func=handle_setstate)


def is_archivable(participation, days):
    return (all_complete_and_valid_condition(participation, None) and
            datetime.datetime.now() >= (participation.updated +
                                        datetime.timedelta(days)))


def handle_setstate(args):
    path = []
    path.append(get_app_location(args.app))
    path.append(args.config)
    session = get_session(os.path.join(*path))
    # Ignore SAWarning, because our data model let SA write a warning
    # and we want to use this in a cron job.
    # Should be temporary until the warning is fixed.
    with warnings.catch_warnings():
        warnings.simplefilter("ignore", category=sa_exc.SAWarning)

        participations = session.query(Participation).filter(
            Participation.participation_state_id == 1).all()
        if not args.execute:
            print "Started in not execute mode."
            print "Show the ids of the archivable participations."
        for participation in participations:
            if not participation.complete:
                print ("autoarchiver: Participation %d doesn't have complete "
                       "value. Ignore it.")  % participation.id
                continue
            if is_archivable(participation, args.days):
                if args.execute:
                    participation.participation_state_id = 2
                else:
                    print participation.id
        if args.execute:
            transaction.commit()
        else:
            print "End"
