# encoding: utf-8
# pylint: skip-file
"""Includes the handle_validate function and its helpers."""
import os.path

from paste.deploy.loadwsgi import appconfig
from pyramid import testing
from pyramid.registry import Registry

from formbar.config import Config, load
from formbar.form import (
    Form,
    Dependency
)
from ringo.lib.helpers import get_app_location, dynamic_import
from ringo.lib.helpers.format import format_date
from ringo.scripts.db import get_session


class DependencyProvider(object):

    def __init__(self, db=None, item=None):
        self.db = db
        self.item = item

    def create_dependency(self):
        dummy_request = testing.DummyRequest()

        return Dependency({},
                          self.db,
                          dummy_request.registry.settings,
                          lambda x: x,
                          self.item,
                          dummy_request.method,
                          "de",
                          dummy_request.registry)


def handle_formvalidate(args, validators=None, provide_validators=None):
    # FIXME: Initialialising the testing modul is currently the only
    # known way to make the settings
    # available in the current_registry call (which is called inside
    # ringo when loading the form).
    config = appconfig('config:%s' % args.config, "main", relative_to='.')
    name = config.context.distribution.project_name
    registry = Registry(name)
    registry.settings = config
    testing.setUp(registry)

    if args.datatype == "participations":
        get_config_form = get_config_form_by_app
        get_data = get_participation_data
    else:
        raise NotImplementedError("Use only datatype 'participations'.")

    config_form = get_config_form(args.app)
    session = get_session_by_app_and_config(args.app, args.config)

    # Import class and filters based on the current
    # application
    data_clazz = dynamic_import("%s.%s" % (args.app, args.clazz))

    data_list = get_data(session, args.app, data_clazz)

    print "Validate ID Errors"
    for data, item in data_list:
        form = get_form_by_config_form(config_form, item, session)
        if validators:
            for validator in validators:
                form.add_validator(validator)
        if provide_validators:
            for validator in provide_validators(item):
                form.add_validator(validator)
        print form.validate(data), item.id, form.get_errors()


def get_config_form_by_app(app):
    config = Config(load(os.path.join(get_app_location(app),
                                      app, "views", "forms",
                                      "participations.xml")))
    config_form = config.get_form("update")
    return config_form


def get_form_by_config_form(config_form, item, session):
    dependencies = DependencyProvider(
        item=item, db=session).create_dependency()
    form = Form(config_form, dependencies=dependencies)
    return form


def get_session_by_app_and_config(app, config):
    path = []
    path.append(get_app_location(app))
    path.append(config)
    session = get_session(os.path.join(*path))
    return session


def get_participation_data(session, app, participation_class):
    participations = session.query(participation_class).all()
    data = []
    for participation in participations:
        pn_dict = {}
        for field in vars(participation_class):
            if field == "participant":
                participant = getattr(participation, field)
                for field_2 in vars(participant):
                    if field_2 == "birthday" and getattr(participant, field_2):
                        value = format_date(getattr(participant, field_2))
                    else:
                        value = getattr(participant, field_2)
                    pn_dict["participant." + field_2] = value
                pn_dict["participant"] = participant
            elif field.endswith(app + "oi"):
                oi = getattr(participation, field)
                if not oi:
                    continue
                for field_2 in vars(oi):
                    pn_dict[app + "oi." + field_2] = getattr(oi, field_2)
            elif field.endswith(app + "ri"):
                ri = getattr(participation, field)
                for field_2 in vars(ri):
                    pn_dict[app + "ri." + field_2] = getattr(ri, field_2)
            elif field in ("planned_end_date", "start_date", "end_date"):
                if getattr(participation, field):
                    value = format_date(getattr(participation, field))
                    pn_dict[field] = value
            elif not field.startswith("_"):
                pn_dict[field] = getattr(participation, field)
        data.append((pn_dict, participation))
    return data
