#!/usr/bin/env python
# encoding: utf-8
import collections
import os.path
from paste.deploy.loadwsgi import appconfig
from pyramid import testing
from pyramid.registry import Registry
from ringo.lib.helpers import get_app_location, dynamic_import
from ringo.scripts.db import get_session, get_appsettings_
from ringo_target.lib.detectsystem import export


def add_target_parser(subparsers, parent):
    t = subparsers.add_parser('target',
                              help='Target administration',
                              parents=[parent])
    sp = t.add_subparsers(help='Target command help')
    export_parser = sp.add_parser('export',
                                  help='Return export from the target system',
                                  parents=[parent])
    export_parser.add_argument('--end-year', dest='year', required=True,
                        help='Set a end year of the export')
    export_parser.set_defaults(func=handle_export_command)
    table_parser = sp.add_parser('table',
                                 help='Return table for target for evel',
                                 parents=[parent])
    table_parser.set_defaults(func=handle_table_command)
    copyfrom_parser = sp.add_parser('copyfrom',
                                 help='Return copyfrom sql for target for eval',
                                 parents=[parent])
    copyfrom_parser.set_defaults(func=handle_copyfrom_command)
    totaltable_parser = sp.add_parser('totaltable',
                                      help=('Return total table for total'
                                            'targets in eval db'),
                                      parents=[parent])
    totaltable_parser.set_defaults(func=handle_totaltable_command)
    totalexport_parser = sp.add_parser('totalexport',
                                       help='Return the totaltarget.',
                                       parents=[parent])
    totalexport_parser.set_defaults(func=handle_totalexport_command)


def handle_export_command(args):
    # Build end year
    end = int(args.year)

    # FIXME: Initialialising the testing modul is currently the only
    # known way to make the settings
    # available in the current_registry call (which is called inside
    # ringo when loading the form).
    config = appconfig('config:%s' % args.config, "main", relative_to='.')
    name = config.context.distribution.project_name
    registry = Registry(name)
    registry.settings = config
    testing.setUp(registry)

    # get get_new_entries from app.view.home
    home = dynamic_import("%s.views.home" % args.app)
    get_new_entries = home.get_new_entries

    # get a session for the application
    path = []
    path.append(get_app_location(args.app))
    path.append(args.config)
    session = get_session(os.path.join(*path))

    # create a DummyRequest with registry and db
    settings = get_appsettings_(args.config)
    DummyRequest = collections.namedtuple("DummyRequest", "registry db")
    dreq = DummyRequest(registry=registry, db=session)

    # export from ringo_target
    print export(dreq, end, get_new_entries)


def handle_totalexport_command(args):
    """Return the totaltarget."""
    settings = get_appsettings_(args.config)
    print settings.get("efa.target.total", 0)


def handle_table_command(args):
    res = []
    res.append("DROP TABLE IF EXISTS eval_targets;")
    res.append("CREATE TABLE eval_targets (")
    res2 = []
    for int_field in ("organisation_id", "year", "month", "entries",
                      "cum_entries", "total_entries", "quota", "cum_quota",
                      "total_quota", "target", "cum_target", "total_target"):
        res2.append("  %s INTEGER" % int_field)
    for text_field in ("color", "cum_color", "total_color"):
        res2.append("  %s TEXT" % text_field)
    res.append(", \n".join(res2))
    res.append(");")
    print "\n".join(res)


def handle_copyfrom_command(args):
    res = """COPY eval_targets (year,month,target,entries,quota,color,
                                cum_target,cum_entries,cum_quota,cum_color,
                                total_target,total_entries,total_quota,
                                total_color)
             FROM STDIN DELIMITER ';' HEADER CSV;"""
    print res


def handle_totaltable_command(args):
    res = []
    res.append("CREATE TABLE eval_totaltargets (")
    res.append("  organisation_id INTEGER,")
    res.append("  totaltarget INTEGER")
    res.append(");")
    print "\n".join(res)
