#!/usr/bin/env python
# encoding: utf-8
"""Handle transfer participation to participant."""
import os.path

from ringo.lib.helpers import get_app_location
from ringo.scripts.db import get_session
import transaction
from efa.model.participation import Participation
from efa.model.participant import Participant


def add_transfer_parser(subparsers, parent):
    """add parser to participations parser"""
    transfer_parser = subparsers.add_parser(
        "transfer_participation", parents=[parent],
        help="Transfer a matching participation to a participant")
    transfer_parser.add_argument("--participation_id", type=int,
                                 help=("id of the participation which should"
                                       " be transfered"))
    transfer_parser.add_argument("--participant_personal_id", type=str,
                                 help=("personal_id of the destinion "
                                       "participant"))
    transfer_parser.set_defaults(func=handle_transfer)


def handle_transfer(args):
    """Handle to transfer a pn to a pt, update journal, check matching_pts."""
    session = get_session_by_app_and_config(args.app, args.config)
    # simple transfer
    transfer_pn_to_pt(session, args.participation_id,
                      args.participant_personal_id)
    transaction.commit()


def get_session_by_app_and_config(app, config):
    path = []
    path.append(get_app_location(app))
    path.append(config)
    session = get_session(os.path.join(*path))
    return session


def transfer_pn_to_pt(session, participation_id, participant_personal_id):
    participation = session.query(Participation).get(participation_id)
    participant = session.query(Participant).filter(
        Participant.personal_id == participant_personal_id).one()
    participation.participant = participant
    session.add(participation)
