# -*- coding: utf8 -*-
from pyramid.renderers import render_to_response
from pyramid.view import view_config
from efa.views.export_participations import access_allowed, prepare_form, \
    get_metadata
from efa.lib.export import whitelist as whitelist_fct
from efa.lib.export import filter_virtual_participation, \
    get_whitelisted_virtual_participations_for
from efa.lib.whitelist import Whitelist
from efa.lib.export_formats.xlsx_template import export_xlsx_template


class FilterConfig(object):

    def __init__(self, entry_filter, gender_filter, start, end,
                 foerderphase):
        self.entry_filter = entry_filter
        self.gender_filter = gender_filter
        self.start = start
        self.end = end
        self.foerderphase = foerderphase

    def filter(self, virtual_participations):
        return filter_virtual_participation(virtual_participations, self)


def get_participation_model(program):
    if program.lower() == "waskiq":
        return "model.participation.WaskiqParticipation"
    else:
        return "model.participation.%sParticipation" % program.upper()


def build_response(request, filters, program, metadata):
    start = str(filters.start)
    end = str(filters.end)
    if start == 'None':
        start = request.registry.settings["efa.project_min_start"]
    if end == 'None':
        end = request.registry.settings["efa.project_max_end"]

    whitelist = Whitelist.from_file(request.registry.settings.get(
        "whitelist_path"))
    whitelisted_virtual_participations = \
        get_whitelisted_virtual_participations_for(
            request,
            program.lower(),
            get_participation_model(program),
            "views.report_durchfuehrungsbericht",
            start, end)
    filtered_virtual_participations = filters.filter(
        whitelisted_virtual_participations)
    result = whitelist_fct(whitelist.fields_for_export,
                           filtered_virtual_participations)
    result.insert(0, whitelist.fields_for_export)
    if request.POST['_submit'] == u"xlsx":
        return export_xlsx_template(request, result, "Teilnehmende", program, metadata,
                            simple_filenames=True)
    return export_xlsx_template(request, result, "Teilnehmende", program, metadata)


def get_filters_from(form):
    filter_type_map = {1: "entry_and_exit_in", 2: "entry_in", 3: "exit_in",
                       4: "entry_or_exit_in", 5: "running_while",
                       -1: "no_filter"}
    gender_map = {0: "only_female", 1: "only_male", None: "both"}
    foerderphase_map = {0: "only_fp1", 1: "only_fp2", None: "both"}
    return FilterConfig(filter_type_map[form.data.get("filter_type")],
                        gender_map.get(form.data.get("gender", "both")),
                        form.data.get('start_date'),
                        form.data.get('end_date'),
                        foerderphase_map.get(form.data.get('foerderphase')))


@view_config(route_name='participants-excelexport',
             renderer='/excelexportform.mako',
             request_method='GET')
def participations_excelexport_form(request):
    access_allowed(request.user)
    form = prepare_form(request, "participants")
    return {'form': form, 'rendered_form': form.render(),
            'exportname': 'Teilnehmende Export (validiert)'}


def participants_excel_export(request, program):
    access_allowed(request.user)

    form = prepare_form(request, "participants")
    form.validate(request.POST)

    if not form.has_errors():
        filters = get_filters_from(form)
        metadata = get_metadata(request, form)
        return build_response(request, filters, program, metadata)
    else:
        return render_to_response('/excelexportform.mako',
                                  {'form': form,
                                   'rendered_form': form.render(),
                                   'exportname': ('Teilnehmende Export '
                                                  '(validiert)')},
                                  request)
