#!/usr/bin/env python
# encoding: utf-8
import pkg_resources
import datetime
from sqlalchemy import or_, asc
from pyramid.view import view_config
from ringo.model.base import get_item_list
from ringo.views.home import (
    index_view as ringo_index_view,
    version_view as ringo_version_view
)
from efa.model.participation import Participation, calc_complete
from ringo_news.views import get_news


def get_reminders(request):
    """Will return a list of participations where the planned exit date
    has not been set has been passed and the enddate is not set"""
    today = datetime.date.today()
    all_participations = request.db.query(Participation)\
        .filter(or_(Participation.planned_end_date < today,
                Participation.planned_end_date == None))\
        .filter(Participation.end_date == None)\
        .filter(Participation.participation_state_id == 1)\
        .order_by(asc(Participation.planned_end_date).nullsfirst())\
        .all()
    # Put loaded participations into a BaseList to be able to filter
    # items based on their permission.
    reminders = get_item_list(request, Participation,
                              user=request.user, items=all_participations)
    # Because the planned_end data is influential to the completeness of
    # the desired fields of the results we reset the completness if
    # it is currently at 100%
    for item in reminders.items:
        if item.complete_ri == 100:
            item.complete = calc_complete(item)
    return reminders.items


def get_has_errors_items(request):
    """Will return the participations, which has errors and
    should be looked at."""
    participations = request.db.query(Participation)\
        .filter(Participation.has_errors == True)\
        .all()
    items = get_item_list(request, Participation, user=request.user,
                          items=participations).items
    return items


@view_config(route_name='home', renderer='/index.mako')
def index_view(request):
    # First call the default view.
    values = ringo_index_view(request)
    # Only get reminders etc. if the user is logged in.
    if request.user:
        values['reminders'] = get_reminders(request)
        values['news'] = get_news(request)
        values['has_errors_items'] = get_has_errors_items(request)
    return values


@view_config(route_name='version', renderer='/version.mako')
def version_view(request):
    # First call the default view.
    values = ringo_version_view(request)
    # Provide version of eFa in the template for all derived
    # applications.
    efa_pkg = pkg_resources.get_distribution('eFa')
    values['efa_version'] = efa_pkg.version
    return values
