import datetime
import logging

from efa.model.journal import Journal
from efa.views.participation import create_oi
from formbar.form import Validator, ValidationException
from pyramid.httpexceptions import HTTPFound
from pyramid.view import view_config
from ringo.lib.helpers import get_action_routename
from ringo.lib.helpers.appinfo import get_app_name
from ringo.lib.renderer.dialogs import InfoDialogRenderer
from ringo.views.base import create as ringo_create
from ringo.views.base import update as ringo_update
from ringo.views.helpers import get_item_from_request

log = logging.getLogger(__name__)

def journal_callback(request, item):
    """Check that the journal has not triggered the 8 hour rule"""
    participant = item.participation.participant

    # If there are already (mandatory) output indicators for the participant,
    # no further tests are necessary
    if not participant.oi \
       and participant.total_journal_duration >= datetime.timedelta(hours=8):
        create_oi(request.user, item.participation)
        request.session['new_oi_8h'] = True
        log.debug('8 hour trigger reached')
    return item


@view_config(route_name=get_action_routename(Journal, 'create'),
             renderer='/default/create.mako',
             permission='create')
def create_(request):
    relation = request.params.get('addrelation')
    if relation is None:
        return ringo_create(request, callback=journal_callback)
    else:
        out = ringo_create(request, callback=journal_callback)
        item = get_item_from_request(request) 

        item.editor = unicode(request.user.profile[0]) 
        # If creating this entry triggered the 8h OI rule in waskiq, redirect
        appname = get_app_name()
        if appname == 'waskiq' and request.POST and 'new_oi_8h' in request.session:
            url = request.route_path("info-8h-oi")
            del request.session['new_oi_8h']
            return HTTPFound(location=url)
        return out


@view_config(route_name='info-8h-oi', renderer='/default/info.mako')
def info_cm_oi(request):
    _ = request.translate
    title = _(u"Please fill in the Output Indicators!")
    body = _(u"With this entry you have reached the 8 hour mark with the "
             "participant. You are now obligated to fill in the Output "
             "Indicators.")
    renderer = InfoDialogRenderer(request, title=title, body=body)
    rvalue = {}
    rvalue['ok_url'] = request.ringo.history.pop(1)
    rvalue['dialog'] = renderer.render([])
    return rvalue


@view_config(route_name=get_action_routename(Journal, 'update'),
             renderer='/default/update.mako',
             permission='update')
def update_(request):
    journal = get_item_from_request(request)
    birthday = journal.participation.participant.birthday
    if birthday is None:
        return ringo_update(request)
    else:
        out = ringo_update(request)

        item = get_item_from_request(request)
        if request.method == "POST":
            item.editor = unicode(request.user.profile[0]) 
        return out

