from pyramid.view import view_config

from efa.model.organisation import Organisation
from ringo.views.base import delete as ringo_delete
from ringo.views.helpers import get_item_from_request
from ringo.views.request import get_return_value, is_confirmed
from ringo.lib.helpers import get_action_routename
from ringo.lib.renderer import InfoDialogRenderer
from ringo.model.user import Usergroup

from ringo.views.base.list_ import set_bundle_action_handler


def cb_organisation_delete(request, item):
    """Will ensure that the usergroup of the organisation will be
    deleted too if the organisation is deleted.

    :request: Current request
    :item: organisation item
    :returns: organisation item

    """
    item.users = []
    org_group = item.group
    # 1. After deletion of the usergroup the users will be member of any
    # organisation we set the GID of the user to None. This ensures that
    # these users are not selectable as standin for other users.
    for user in org_group.members:
        user.gid = None
        default_group = request.db.query(Usergroup).filter(Usergroup.id == user.default_gid).one()
        default_group.members = [user]

    # 2. To be able to delete the usergroup of the organisation we need
    # to remove all users to avoid FK constraint errors.
    org_group.members = []
    # 3. Finally delete the usergroup of the organisation.
    request.db.delete(org_group)
    return item


def _cannot_delete_org_dialog(request, item):
    _ = request.translate
    if len(item.users) > 0:
        mapping = {'item': item}
        title = _("Can not delete organisation '${item}'",
                  mapping=mapping)
        body = _("The organisation '${item}' still has assigned users. "
                 "Please transfer them first to another organisation",
                 mapping=mapping)
        request.db.rollback()
        renderer = InfoDialogRenderer(request, title, body)
        rvalue = {}
        rvalue['item'] = item
        rvalue['clazz'] = item.__class__
        ok_url = request.route_path(get_action_routename(Organisation, 'list'))
        rvalue['dialog'] = renderer.render(ok_url)
        return rvalue
    return None


@view_config(route_name=get_action_routename(Organisation, 'delete'),
             renderer='/default/confirm.mako',
             permission='delete')
def delete_(request):
    item = get_item_from_request(request)
    if request.POST:
        dialog = _cannot_delete_org_dialog(request, item)
        if dialog is not None:
            return dialog
    return ringo_delete(request, callback=cb_organisation_delete)


def _org_handle_delete_request(request, items, callback):
    from ringo.views.base.delete import _handle_delete_request
    if request.POST and is_confirmed(request):
        for item in items:
            dialog = _cannot_delete_org_dialog(request, item)
            if dialog is not None:
                return dialog
    return _handle_delete_request(request, items, callback=cb_organisation_delete)

set_bundle_action_handler("delete", _org_handle_delete_request, "organisations")
