# -*- coding: utf-8 -*-

from pyramid.i18n import get_locale_name
from formbar.form import Form
import collections
import re
import datetime
from pyramid.httpexceptions import HTTPUnauthorized
import ringo.lib.security as security
from efa.lib.validators import strip_prefix
from efa.lib.report import filter_broken_participations, filter_date_range
from ringo.lib.i18n import locale_negotiator
from ringo.lib.form import get_form_config_from_file, get_eval_url
from ringo.lib.helpers import get_app_url

Dates = collections.namedtuple('Dates', 'fromdate todate')
is_dateformat = re.compile("^\d{4}-\d{2}-\d{2}$").match


def default_dates(request):
    # For the first three month of the year the default period for the
    # evaluation should be the prior year. See issue967@intevation
    # See issue1401@intevation for this.

    fixed_begin = None
    if request.registry.settings.get("efa.report.start"):
        fixed_begin = datetime.datetime.strptime(
            request.registry.settings.get("efa.report.start"), "%Y-%m-%d")

    this_year = datetime.datetime.now().year
    this_month = datetime.datetime.now().month
    threshold = int(request.registry.settings.get("efa.report.monththreshold",
                                                  4))
    if this_month <= threshold:
        this_year -= 1
    this_year = int(request.registry.settings.get("efa.report.defaultyear",
                                                  this_year))
    if fixed_begin:
        begin = fixed_begin
    else:
        begin = datetime.date(this_year, 1, 1)
    end = datetime.date(this_year, 12, 31)
    return (begin, end)


def prepare_report(request, formconfig=None):
    if not security.has_role(request.user, "beneficiary"):
        raise HTTPUnauthorized

    if formconfig is None:
        formconfig = get_form_config_from_file("efa",
                                               "reports.xml",
                                               "entryandexit")
    form = Form(formconfig, None, request.db,
                translate=request.translate,
                request=request,
                renderers={},
                change_page_callback={'url': 'set_current_form_page'},
                locale=locale_negotiator(request),
                url_prefix=get_app_url(request),
                eval_url=get_eval_url(),
                csrf_token=request.session.get_csrf_token())

    if request.POST:
        form.validate(request.params)
        values = {}
    else:
        begin, end = default_dates(request)
        locale_name = get_locale_name(request)
        if locale_name.startswith("de"):
            begin = begin.strftime("%d.%m.%Y")
            end = end.strftime("%d.%m.%Y")
        else:
            begin = begin.strftime("%Y-%m-%d")
            end = end.strftime("%Y-%m-%d")
        values = {"start_date": begin,
                  "end_date": end}

    search = form.render(values=values)
    return {'search': search, 'form': form}


def evaluate_1(participations, to_bmas, start, end):
    """A simple report generator"""
    entries = filter_date_range(participations, "start_date", start, end)
    exits = filter_broken_participations(participations)
    exits = filter_date_range(exits, "end_date", start, end)

    out = {"26": 0, "27": 0, "28": 0, "29": 0,
           "entries": len(entries),
           "exits": len(exits)}

    for participation in exits:
        bmas = to_bmas(strip_prefix(participation))
        for field in ("26", "27", "28", "29"):
            if bmas.get(field):
                out[field] += 1
    return out
